pub struct Error(/* private fields */);Expand description
Indicates a problem while constructing a client.
Examples
use google_cloud_gax::client_builder::Error as Error;
use examples::Client; // Placeholder for examples
let client = match Client::builder().build().await {
Ok(c) => c,
Err(e) if e.is_default_credentials() => {
println!("error during client initialization: {e}");
println!("troubleshoot using https://cloud.google.com/docs/authentication/client-libraries");
return Err(e);
}
Err(e) => {
println!("error during client initialization {e}");
return Err(e);
}
};Implementations
Sourceimpl Error
impl Error
Sourcepub fn is_default_credentials(&self) -> bool
pub fn is_default_credentials(&self) -> bool
If true, the client could not initialize the default credentials.
Sourcepub fn is_transport(&self) -> bool
pub fn is_transport(&self) -> bool
If true, the client could not initialize the transport client.