#[non_exhaustive]pub enum StatusDetails {
BadRequest(BadRequest),
DebugInfo(DebugInfo),
ErrorInfo(ErrorInfo),
Help(Help),
LocalizedMessage(LocalizedMessage),
PreconditionFailure(PreconditionFailure),
QuotaFailure(QuotaFailure),
RequestInfo(RequestInfo),
ResourceInfo(ResourceInfo),
RetryInfo(RetryInfo),
Other(Any),
}Expand description
The type of details associated with Status.
Google cloud RPCs often return a detailed error description. This details can be used to better understand the root cause of the problem. Detailed information about the error.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
BadRequest(BadRequest)
Describes violations in a client request.
See BadRequest for more information.
DebugInfo(DebugInfo)
Describes additional debugging info.
See DebugInfo for more information.
ErrorInfo(ErrorInfo)
Describes the cause of the error with structured details.
See ErrorInfo for more information.
Help(Help)
Provides links to documentation or for performing an out of band action.
See Help for more information.
LocalizedMessage(LocalizedMessage)
Provides a localized error message that is safe to return to the user.
See LocalizedMessage for more information.
PreconditionFailure(PreconditionFailure)
Describes what preconditions have failed.
See PreconditionFailure for more information.
QuotaFailure(QuotaFailure)
Describes a single quota violation.
See QuotaFailure for more information.
RequestInfo(RequestInfo)
Contains metadata about the request that clients can attach when filing a bug.
See RequestInfo for more information.
ResourceInfo(ResourceInfo)
Describes the resource that is being accessed.
See ResourceInfo for more information.
RetryInfo(RetryInfo)
Describes when the clients can retry a failed request.
See RetryInfo for more information.
Other(Any)
Other details (represented as Any).